#! /usr/bin/env python
#
# Copyright INRA-URGI 2009-2010
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
# 
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
#
"""
Change the name of a tag
"""

import os
import random
from optparse import OptionParser
from SMART.Java.Python.structure.TranscriptContainer import TranscriptContainer
from SMART.Java.Python.misc.Progress import Progress
from commons.core.writer.MySqlTranscriptWriter import MySqlTranscriptWriter
from commons.core.writer.Gff3Writer import Gff3Writer


if __name__ == "__main__":
    
    # parse command line
    description = "Change Tag Name v1.0.1: Change the name of tag of a list of transcripts. [Category: Data Modification]"

    parser = OptionParser(description = description)
    parser.add_option("-i", "--input",       dest="inputFileName",  action="store",                      type="string", help="input file [compulsory] [format: file in transcript format given by -f]")
    parser.add_option("-f", "--inputFormat", dest="inputFormat",    action="store",                      type="string", help="format of the input file [compulsory] [format: transcript file format]")
    parser.add_option("-o", "--output",      dest="outputFileName", action="store",                      type="string", help="output file [compulsory] [format: output file in GFF3 format]")
    parser.add_option("-t", "--tag",         dest="tag",            action="store",                      type="string", help="name of the tag to change [compulsory] [format: string]")
    parser.add_option("-n", "--name",        dest="name",           action="store",                      type="string", help="new name for the tag [compulsory] [format: string]")
    parser.add_option("-y", "--mysql",       dest="mysql",          action="store_true", default=False,                 help="mySQL output [format: bool] [default: false]")    
    parser.add_option("-v", "--verbosity",   dest="verbosity",      action="store",      default=1,      type="int",    help="trace level [format: int] [default: 1]")
    parser.add_option("-l", "--log",         dest="log",            action="store_true", default=False,                 help="write a log file [format: bool] [default: false]")
    (options, args) = parser.parse_args()

    if options.log:
        logHandle = open("%s.log" % options.outputFileName, "w")

    # create parser and writer(s)
    parser      = TranscriptContainer(options.inputFileName, options.inputFormat, options.verbosity)
    tmpFileName = "tmpTranscriptFile%d.gff3" % (random.randint(0, 100000))
    writer      = Gff3Writer(tmpFileName, options.verbosity)
    if options.mysql:
        mysqlWriter = MySqlTranscriptWriter(options.outputFileName, options.verbosity)
    outputData = {}
        
    # process transcripts
    progress = Progress(parser.getNbTranscripts(), "Printing transcripts %s" % (options.inputFileName), options.verbosity)
    for transcript in parser.getIterator():
        if options.tag in transcript.tags:
            value = transcript.tags[options.tag]
            del transcript.tags[options.tag]
            transcript.tags[options.name] = value
        writer.addTranscript(transcript)
        if options.mysql:
            mysqlWriter.addTranscript(transcript)
        progress.inc()
    progress.done()
    parser.transcriptListParser.close()

    writer.write()
    writer.close()

    if options.mysql:
        mysqlWriter.write()

    os.rename(tmpFileName, options.outputFileName)
